/* PSPP - computes sample statistics.
   Copyright (C) 1997 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   The author may be contacted at <pfaffben@pilot.msu.edu> on the
   Internet, or as Ben Pfaff, 12167 Airport Rd, DeWitt MI 48820, USA
   through more mundane means. */

#if !common_h
#define common_h 1

#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <limits.h>
#include <float.h>

/* Constants. */

/* Endianness variable. */
#if ENDIAN==UNKNOWN
/* BIG or LITTLE, depending on this machine's endianness, as detected
   at program startup. */
extern int endian;
#else /* we know the endianness at compile time */
#define endian ENDIAN		/* let the compiler optimize everything out */
#endif /* we know the endianness at compile time */

/* "Second lowest" value for a flt64; that is, (-FLT64_MAX) + epsilon. */
#ifdef SECOND_LOWEST_VALUE	/* from pref.h */
#define second_lowest_value SECOND_LOWEST_VALUE
#else
extern double second_lowest_value;
#endif

/* System-missing value: the largest negative value. */
#define SYSMIS -DBL_MAX

/* NOT_LONG and NOT_INT are used when we want a "missing value" for an
   integer-type variable.  Just assume that those particular values
   will never crop up... */
#define NOT_LONG LONG_MAX
#define NOT_INT INT_MAX

/* Definition of the max length of a short string value, generally
   eight characters.  */
#define MAX_SHORT_STRING ((SIZEOF_DOUBLE)>=8 ? ((SIZEOF_DOUBLE)+1)/2*2 : 8)
#define MIN_LONG_STRING (MAX_SHORT_STRING+1)

/* Variables. */

/* argv[0] with stripped leading directories. */
extern char *pgmname;

/* Whether FINISH. has been executed. */
extern int finished;

/* The current date in the form DD MMM YYYY. */
extern char curdate[];

/* The name of the procedure currently executing, if any. */
extern char *cur_proc;

/* Whether we're dropping down to interactive mode immediately because
   we hit end-of-file unexpectedly (or whatever). */
extern int start_interactive;

#if HAVE_LIBHISTORY
/* Name of interactive command-line history file. */
char *history_file;
#endif

/* Functions. */

void *xmalloc (size_t size);
void *xrealloc (void *ptr, size_t size);
char *xstrdup (char *s);

/* Brokenness. */

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif

#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#ifndef RAND_MAX
#undef rand
#undef srand
#define rand ansi_rand
#define srand ansi_srand
#define RAND_MAX 32767
#define BROKEN_RAND 1
#endif

#endif /* common_h */
