/* PSPP - computes sample statistics.
   Copyright (C) 1997 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   The author may be contacted at <pfaffben@pilot.msu.edu> on the
   Internet, or as Ben Pfaff, 12167 Airport Rd, DeWitt MI 48820, USA
   through more mundane means. */

#ifndef EXTERN
#error EXTERN must be defined.
#endif

/* som output driver extension record. */
typedef struct
  {
    /* All of these below are in device units. */
    int cp_x, cp_y;		/* Current position. */
    int width[SLIN_COUNT];	/* Widths of vertical lines. */
    int height[SLIN_COUNT];	/* Heights of horizontal lines. */
    int font_height;		/* Default font character height. */
    int prop_em_width;		/* Proportional font em width. */
    int fixed_width;		/* Fixed-pitch font character width. */
  }
som_driver_ext;

/* Virtual local variables (why can't all compilers have nested
   functions?!) */
EXTERN struct
  {
    outp_driver *d;		/* Driver. */
    int options;		/* som_submission_form.options */
    a_string title;		/* Table's title. */
    int title_height;		/* Table height. */
    som_driver_ext *ext;	/* Driver extension. */
    som_table *t;		/* Pointer to table in use. */
    som_table local_t;		/* Local copy of passed_t if necessary. */
    int mh, mv;			/* Max dimensions of any cell not in header. */
    int hh, hv;			/* Dimensions of header rows/columns. */
    int htv;			/* Height of top header rows. */
    int th;			/* Width of table with headers. */
    int *ch, nch, mch;		/* Indicate columns across the pages. */
    int mpw;			/* For crushed tables, maximum page width. */
    int type;			/* Table type. */
    int digit_space;		/* Horizontal space for row labels. */
  }
som;

/* Arena associated with the table currently being constructed. */
EXTERN arena *curtab_arena;

/* Stack of table arenas. */
EXTERN arena **arena_stack;
EXTERN int n_arena_stack, m_arena_stack;

/* Some columns represent rules, some represent cells. */
#define RULE_COLUMN 0
#define CELL_COLUMN 1

/* Returns column type for column C. */
#define COLUMN_TYPE(C) 				\
	((C) % 2)

/* Column type predicates for column C. */
#define IS_RULE_COLUMN(C) 			\
	(COLUMN_TYPE (C) == RULE_COLUMN)
#define IS_CELL_COLUMN(C) 			\
	(COLUMN_TYPE (C) == CELL_COLUMN)

/* Some rows represent rules, some represent cells. */
#define RULE_ROW 0
#define CELL_ROW 1

/* Returns row type for column C. */
#define ROW_TYPE(R) 				\
	((R) % 2)

/* Row type predicates for row R. */
#define IS_RULE_ROW(R) 				\
	(ROW_TYPE (R) == RULE_ROW)
#define IS_CELL_ROW(R) 				\
	(ROW_TYPE (R) == CELL_ROW)

/* Calculates subrow corresponding to the top subrow in row R. */
#define TOP_SUBROW(R) 				\
	((som.nch + 1) * (R) + 1)

/* Calculates subrow corresponding to the bottom subrow in row R.
   This is equivalent to som.nch*((r)+1) + (r) but avoids evaluating R
   twice. */
#define BOTTOM_SUBROW(R) 			\
	(TOP_SUBROW (R) + som.nch - 1)

/* Calculates subrow for the rule separating rows (R-1) and R. */
#define SUBROW_BEFORE(R) 			\
	(TOP_SUBROW (R) - 1)

/* Calculates subrow for the rule separating rows R and (R+1). */
#define SUBROW_AFTER(R) 			\
	(BOTTOM_SUBROW (R) + 1)

/* For a subrow that indexes a rule, returns 0; otherwise, returns the
   1-based subrow index relative to the row. */
#define SUBROW_WITHIN_ROW(SR) 			\
	((SR) % (som.nch + 1))

/* Returns 1 if the subrow indexes a rule, 0 if it indexes a subrow
   proper. */
#define SUBROW_IS_RULE(SR) 			\
	(SUBROW_WITHIN_ROW (SR) == 0)

/* Returns row corresponding to subrow SR. */
#define SUBROW_TO_ROW(SR) 			\
	(SUBROW_IS_RULE(SR)			\
	 ? ((SR) / (som.nch + 1) * 2)		\
	 : ((SR) - 1) / (som.nch + 1) * 2 + 1)

/* Returns height of subrow SR. */
#define SUBROW_HEIGHT(SR) 			\
	(t->h[SUBROW_TO_ROW (SR)])

int som_init_driver (outp_driver *);
void som_draw_title (int px, int py, int test);
void som_dump_page (int x1, int y1, int x2, int y2, int px, int py, int trim);
void som_get_table_size (outp_driver *, som_table *);
void som_dump_page (int x1, int y1, int x2, int y2, int px, int py, int trim);
int som_internal_eject_page (outp_driver *);
