/* PSPP - computes sample statistics.
   Copyright (C) 1997 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   The author may be contacted at <pfaffben@pilot.msu.edu> on the
   Internet, or as Ben Pfaff, 12167 Airport Rd, DeWitt MI 48820, USA
   through more mundane means. */

#include <config.h>
#include "common.h"
#include "error.h"
#include "tokens.h"
#include "var.h"

int
cmd_split_file (void)
{
  match_id (SPLIT);
  match_id (FILE);
  if (match_id (OFF))
    {
      if (token != '.')
	return msg (SE, _("End of command expected."));

      default_dict.n_splits = 0;
      free (default_dict.splits);
    }
  else
    {
      variable **v;
      int n, i;

      match_tok (BY);
      if (!parse_variables (NULL, &v, &n, PV_NO_DUPLICATE))
	return 0;
      if (token != '.')
	return msg (SE, _("End of command expected."));

      /* Put all the info in a single array to save
         double-dereferencing. */
      default_dict.splits = xmalloc ((n + 1) * sizeof (split));
      default_dict.n_splits = n;
      for (i = 0; i < n; i++)
	{
	  default_dict.splits[i].v = v[i];
	  default_dict.splits[i].type = v[i]->type;
	  default_dict.splits[i].fv = v[i]->fv;
	  default_dict.splits[i].width = v[i]->width;
	}
      default_dict.splits[n].type = -1;
      free (v);
    }
  return 1;
}
